# -*- coding: utf-8 -*-
# Generated by Django 1.11.24 on 2019-10-02 20:29


from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0076_auto_20190918_2037'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='enterprisecustomerentitlement',
            name='enterprise_customer',
        ),
        migrations.RemoveField(
            model_name='historicalenterprisecustomerentitlement',
            name='enterprise_customer',
        ),
        migrations.RemoveField(
            model_name='historicalenterprisecustomerentitlement',
            name='history_user',
        ),
        migrations.DeleteModel(
            name='EnterpriseCustomerEntitlement',
        ),
        migrations.DeleteModel(
            name='HistoricalEnterpriseCustomerEntitlement',
        ),
    ]
